/* imagetypes.h:
 *
 * supported image types and the imagetypes array declaration.  when you
 * add a new image type, only the makefile and this header need to be
 * changed.
 *
 * jim frost 10.15.89
 */

#ifndef __IMAGETYPES_H__
#define __IMAGETYPES_H__

Image *niffLoad(char *, char *, unsigned int);
Image *facesLoad(char *, char *, unsigned int);
Image *pbmLoad(char *, char *, unsigned int);
Image *sunRasterLoad(char *, char *, unsigned int);
Image *gifLoad(char *, char *, unsigned int);
Image *rleLoad(char *, char *, unsigned int);
Image *xwdLoad(char *, char *, unsigned int);
Image *vffLoad(char *, char *, unsigned int);
Image *xbitmapLoad(char *, char *, unsigned int);
Image *xpixmapLoad(char *, char *, unsigned int);
Image *fbmLoad(char *, char *, unsigned int);
Image *pcxLoad(char *, char *, unsigned int);
Image *imgLoad(char *, char *, unsigned int);
Image *macLoad(char *, char *, unsigned int);
Image *cmuwmLoad(char *, char *, unsigned int);
Image *mcidasLoad(char *, char *, unsigned int);
#if 0
Image *pdsLoad();
#else
Image *vicarLoad(char *, char *, unsigned int);
#endif
#ifdef HAVE_LIBJPEG
Image *jpegLoad(char *, char *, unsigned int);
#endif
#ifdef HAVE_LIBTIFF
Image *tiffLoad(char *, char *, unsigned int);
#endif
#ifdef HAVE_LIBPNG
Image *pngLoad(char *, char *, unsigned int);
#endif

int niffIdent(char *, char *);
int facesIdent(char *, char *);
int pbmIdent(char *, char *);
int sunRasterIdent(char *, char *);
int gifIdent(char *, char *);
int rleIdent(char *, char *);
int xwdIdent(char *, char *);
int vffIdent(char *, char *);
int xbitmapIdent(char *, char *);
int xpixmapIdent(char *, char *);
int fbmIdent(char *, char *);
int pcxIdent(char *, char *);
int imgIdent(char *, char *);
int macIdent(char *, char *);
int cmuwmIdent(char *, char *);
int mcidasIdent(char *, char *);
#if 0
int pdsIdent();
#else
int vicarIdent(char *, char *);
#endif
#ifdef HAVE_LIBJPEG
int jpegIdent(char *, char *);
#endif
#ifdef HAVE_LIBTIFF
int tiffIdent(char *, char *);
#endif
#ifdef HAVE_LIBPNG
int pngIdent(char *, char *);
#endif

void niffDump(Image *, char *, char *, int);
#ifdef HAVE_LIBJPEG
void jpegDump(Image *, char *, char *, int);
#endif
#ifdef HAVE_LIBTIFF
void tiffDump(Image *, char *, char *, int);
#endif
void pbmDump(Image *, char *, char *, int);

/* some of these are order-dependent
 */

struct {
  int    (*identifier)(char fullname[BUFSIZ], char *name); /* print out image info if this kind of image */
  Image *(*loader)(char fullname[BUFSIZ], char *name, unsigned int verbose);     /* load image if this kind of image */
  void   (*dumper)(Image *image, char *optptr, char *filename, int verbose);     /* dump image of this kind */
  char  *type;            /* image type name */
  char  *name;            /* name of this image format */
} ImageTypes[] = {
  {niffIdent,      niffLoad,      niffDump,    "niff",      "Native Image File Format (NIFF)"},
  {sunRasterIdent, sunRasterLoad, NULL,        "sunraster", "Sun Rasterfile"},
  {gifIdent,       gifLoad,       NULL,        "gif",       "GIF Image"},
#ifdef HAVE_LIBJPEG
  {jpegIdent,      jpegLoad,      jpegDump,    "jpeg",      "JFIF-style JPEG Image"},
#endif
#ifdef HAVE_LIBTIFF
  {tiffIdent,      tiffLoad,      tiffDump,    "tiff",      "TIFF image"},
#endif
#ifdef HAVE_LIBPNG
  {pngIdent,       pngLoad,       NULL,        "png",       "PNG image"},
#endif
  {fbmIdent,       fbmLoad,       NULL,        "fbm",       "FBM Image"},
  {cmuwmIdent,     cmuwmLoad,     NULL,        "cmuraster", "CMU WM Raster"},
  {pbmIdent,       pbmLoad,       pbmDump,     "pbm",       "Portable Bit Map (PBM, PGM, PPM)"},
  {facesIdent,     facesLoad,     NULL,        "faces",     "Faces Project"},
  {rleIdent,       rleLoad,       NULL,        "rle",       "Utah RLE Image"},
  {xwdIdent,       xwdLoad,       NULL,        "xwd",       "X Window Dump"},
  {vffIdent,       vffLoad,       NULL,        "vff",       "Sun Visualization File Format"},
  {mcidasIdent,    mcidasLoad,    NULL,        "mcidas",    "McIDAS areafile"},
#if 0
  {pdsIdent,       pdsLoad,       NULL,        "pds",       "PDS/VICAR Image"},
#else
  {vicarIdent,     vicarLoad,     NULL,        "vicar",     "VICAR Image"},
#endif
  {pcxIdent,       pcxLoad,       NULL,        "pcx",       "PC Paintbrush Image"},
  {imgIdent,       imgLoad,       NULL,        "gem",       "GEM Bit Image"},
  {macIdent,       macLoad,       NULL,        "macpaint",  "MacPaint Image"},
  {xpixmapIdent,   xpixmapLoad,   NULL,        "xpm",       "X Pixmap"},
  {xbitmapIdent,   xbitmapLoad,   NULL,        "xbm",       "X Bitmap"},
  {NULL,           NULL,          NULL,         NULL,        NULL}
};

void dumpImage (Image *image, char *type, char *filename, int verbose);

#endif /* __IMAGETYPES_H__ */
